<?php
// $Id: customfilter.admin.inc,v 1.1.2.8.2.3 2009/06/26 00:53:35 kiam Exp $

/**
 * @file
 * Administration page callbacks for Custom filter.
 */
 
/**
 * @addtogroup customfilter
 * @{
 */

/***************************************************************************** 
 * Menu callbacks / form builders, submit/validate functions.
 ****************************************************************************/

/**
 * Return the filter deleting form.
 */ 
function customfilter_filter_delete($form_state, $fid) {
  $filter = customfilter_get_filter($fid, '*');
  $rules = customfilter_get_rules($fid);
  
  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $fid,
  );
  
  $message = t('Are you sure you want to delete %name filter?', array('%name' => $filter['name']));
  
  $msg_text = "<p>". $filter->description ."</p>";
  
  if (count($rules)) {
    $msg_text .= " <p>". t('This filter has replacement rules; if you delete the filter, the replacement rules will be deleted too.') ."</p>";
  }
  
  $msg_text .= " <p>". t('This action cannot be undone.') ."</p>";
  
  return confirm_form(
    $form, $message,
    'admin/settings/customfilter',
    $msg_text,
    t('Delete')
  );
}

/**
 * Submission callback for the filter deleting form.
 */ 
function customfilter_filter_delete_submit($form, &$form_state) {
  $rules = customfilter_get_rules($form_state['values']['fid']);
  
  foreach ($rules as $rule) {
    customfilter_delete_rule($rule['rid']);
  }
  
  db_query('DELETE FROM {customfilter_filters} WHERE fid = %d', $fid);
  
  $form_state['redirect'] = 'admin/settings/filters/customfilter';
}

/**
 * Return the filter editing form.
 */
function customfilter_filter_edit($form_state, $op, $fid = 0) {
  if ($op == 'edit') {
    $item = customfilter_get_filter($fid, '*');
  }
  elseif ($op == 'add') {
    $fid = (integer) db_result(db_query("SELECT MAX(fid) FROM {customfilter_filters}")) + 1;
    $item = array(
      'fid' => $fid,
      'name' => "Filter #{$fid}",
      'cache' => 1,
      'description' => '',
      'shorttips' => '',
      'longtips' => '',
    );
  }
   
  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $item['fid'],
  );
  
  $form['operation'] = array(
    '#type' => 'value',
    '#value' => $op,
  );
  
  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Name'),
    '#description' => t('The name of the filter.'),
    '#default_value' => $item['name'],
    '#required' => TRUE,
  );
  
  $form['cache'] = array(
    '#type' => 'checkbox',
    '#title' => t('Cache'),
    '#description' => t('If checked, the content will be cached.'),
    '#default_value' => $item['cache'],
  );
  
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('A description of the purpose of this filter.'),
    '#default_value' => $item['description'],
  );
  
  $form['shorttips'] = array(
    '#type' => 'textarea',
    '#title' => t('Tips (short)'),
    '#default_value' => $item['shorttips'],
    '#rows' => 5,
  );
  
  $form['longtips'] = array(
    '#type' => 'textarea',
    '#title' => t('Tips (full)'),
    '#default_value' => $item['longtips'],
    '#rows' => 20,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save'),
  );
  
  return $form;
}
 
/**
 * Submission callback for the filter editing form.
 */
function customfilter_filter_edit_submit($form, &$form_state) {
  switch ($form_state['values']['operation']) {
    case 'edit':
      drupal_write_record('customfilter_filters', $form_state['values'], 'fid');
      break;
    
    case 'add':
      drupal_write_record('customfilter_filters', $form_state['values']);
      break;
  }
  
  $form_state['redirect'] = 'admin/settings/filters/customfilter';
}
 
/**
 * Return the list of replacement rules form.
 */
function customfilter_rules(&$form_state, $fid, $rid = 0) {
  $form['rules'] = _customfilter_rules_tree_form(customfilter_get_rules($fid, $rid), 0);
  
  if (element_children($form)) {
    $form['fid'] = array(
      '#type' => 'hidden',
      '#value' => $fid,
    );
    
    $form['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Save configuration'),
    );
  }
  else {
    $form['empty_rule'] = array(
      '#value' => t('There are no replacement rules yet.')
    );
  }
  
  return $form;
}

/**
 * Submission callback for the list of replacement rules form.
 */
function customfilter_rules_submit($form, &$form_state) {
  foreach ($form_state['values']['rules'] as $rule) {
    drupal_write_record('customfilter_rules', $rule, 'rid');
  }
  
  $form_state['redirect'] = "admin/settings/filters/customfilter/{$form_state['values']['fid']}";
}

/**
 * Return the replacement rule delete form.
 */
function customfilter_rule_delete($form_state, $rid) {
  $rule = db_fetch_object(db_query("SELECT * FROM {customfilter_rules} WHERE rid = %d", $rid));
  $subrule = db_fetch_object(db_query("SELECT * FROM {customfilter_rules} WHERE prid = %d", $rid));
  
  $form['rid'] = array(
    '#type' => 'value',
    '#value' => $rid,
  );
  
  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $rule->fid,
  );
  
  $message = t(
    'Are you sure you want to delete this replacement rule: %name?',
    array('%name' => $rule->name)
  );
  
  $textorcode = ($rule->code)? t('Code'): t('Text');
    
  $msg_text = 
    "<p>{$rule->description}</p>" 
    .'<h3>'. t('Pattern') .'</h3>'
    ."<pre>{$rule->pattern}</pre>"
    .'<h3>'. t('Replacement') .' '. $textorcode .'</h3>'
    .'<pre>'. htmlspecialchars($rule->replacement) .'</pre>';
  
  if ($subrule) {
    $msg_text .= " <p>". t('This rule has subrules. If you delete this, they will be deleted too.') ."</p>" . drupal_get_form('customfilter_rules', $rule->fid, $rid, FALSE);
  }

  $msg_text .= " <p>". t('This action cannot be undone.') ."</p>";
    
  return confirm_form(
    $form, $message,
    "admin/settings/customfilter/{$rule->fid}",
    $msg_text,
    t('Delete')
  );
}

/**
 * Submission callback for the replacement rule delete form.
 */
function customfilter_rule_delete_submit($form, &$form_state) {
  customfilter_delete_rule($form_state['values']['rid']);
  
  $form_state['redirect'] = "admin/settings/filters/customfilter/{$form_state['values']['fid']}";
}

/**
 * Return the replacement rule edit form.
 */
function customfilter_rule_edit($form_state, $op, $fid, $rid = 0) {
  if ($op == 'edit') {
    $item = customfilter_get_rule($rid, '*');
  }
  elseif ($op == 'add') {
    $item = array(
      'rid' => (integer) db_result(db_query("SELECT MAX(rid) FROM {customfilter_rules}")) + 1,
      'prid' => $rid,
      'fid' => $fid,
      'name' => '$1',
      'description' => '',
      'matches' => 1,
      'pattern' => '/regex/i',
      'replacement' => 'Regular Expressions',
      'code' => 0,
      'weight' => 0,
    );
  }
  
  $matchopt = drupal_map_assoc(range(0, 99));
  
  $form['rid'] = array(
    '#type' => 'value',
    '#value' => $item['rid'],
  );
  
  $form['fid'] = array(
    '#type' => 'value',
    '#value' => $fid,
  );
  
  $form['prid'] = array(
    '#type' => 'value',
    '#value' => $item['prid'],
  );
  
  $form['operation'] = array(
    '#type' => 'value',
    '#value' => $op,
  );
  
  if ($item['prid'] != 0) {
    $form['matches'] = array(
      '#type' => 'select',
      '#title' => t('# Match'),
      '#description' => t('n-th matched substring in parent rule. This replacement rule will replace only for that substring.'),
      '#options' => $matchopt,
      '#default_value' => $item['matches'],
    );
  }
  
  $form['name'] = array(
    '#type' => 'textfield',
    '#description' => t('The name of this replacement rule.'),
    '#title' => t('Name'),
    '#default_value' => $item['name'],
    '#required' => TRUE,
  );
  
  $form['pattern'] = array(
    '#type' => 'textarea',
    '#title' => t('Pattern'),
    '#description' => t('The regular expression to use. Look at <a href="http://www.php.net/manual/en/regexp.reference.php">Regular Expression Details</a> for more help.'),
    '#default_value' => $item['pattern'],
    '#rows' => 3,
  );
  
  $form['code'] = array(
    '#type' => 'checkbox',
    '#title' => t('PHP Code'),
    '#description' => t('If selected, the replacement text is PHP code.'),
    '#default_value' => $item['code'],
  );
  
  $form['replacement']['replacement'] = array(
    '#type' => 'textarea',
    '#title' => t('Replacement text'),
    '#description' => t('The replacement text that will replace the matched string. Use $n (i.e. $1, $25) or ${n} (i.e. ${1}, ${25}), with n range from 0 to 99, to get the n-th original strings matched ($0 represents the entire matched string). If you set the <strong>PHP Code</strong>, you can enter replaced text with some PHP Code. n-th matched string will be provided in $matches[n], and there will be a global variable named $vars you can use it for your own purpose. PHP code must contain a return statement.'),
    '#default_value' => $item['replacement'],
    '#rows' => 16,
  );
  
  if (module_exists('token')) {
    $form['replacement']['token'] = array(
      '#type' => 'fieldset',
      '#title' => t('Token help'),
      '#description' => t('The replacement text can contain some tokens that will be replaced at runtime with their actual value.'),
      '#collapsed' => TRUE,
      '#collapsible' => TRUE,
    );
    
    $form['replacement']['token']['global-help'] = array(
      '#value' => theme('token_help', 'global'),
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    );
    
    $form['replacement']['token']['node-help'] = array(
      '#value' => theme('token_help', 'node'),
      '#prefix' => '<div>',
      '#suffix' => '</div>',
    );
  }
  
  $form['weight'] = array(
    '#title' => t('Weight'),
    '#default_value' => $item['weight'],
  );
  
  $form['description'] = array(
    '#type' => 'textarea',
    '#title' => t('Description'),
    '#description' => t('The description of this rule.'),
    '#default_value' => $item['description'],
  );
  
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save Configuration'),
  );
  
  return $form;
}
 
/**
 * Submission callback for the replacement rule edit form.
 */
function customfilter_rule_edit_submit($form, &$form_state) {
  switch ($form_state['values']['operation']) {
    case 'edit':
      drupal_write_record('customfilter_rules', $form_state['values'], 'rid');
      break;
    
    case 'add':
      drupal_write_record('customfilter_rules', $form_state['values']);
      break;
  }
  
  $form_state['redirect'] = "admin/settings/filters/customfilter/{$form_state['values']['fid']}";
}

/** 
 * Module settings page.
 */
function customfilter_settings() {
  $header = array(
    t('Name'),
    t('Description'),
    array('data' => t('Operations'), 'colspan' => '2'),
  );
   
  $rows = array();
  
  foreach (customfilter_get_filters() as $filter) {
    $rows[] = array(
      l(t($filter['name']), "admin/settings/filters/customfilter/{$filter['fid']}"),
      '<em>'. t($filter['description']) .'</em>',
      l(t('edit'), "admin/settings/filters/customfilter/{$filter['fid']}/edit"),
      l(t('delete'), "admin/settings/filters/customfilter/{$filter['fid']}/delete"),
    );
  }
  
  $table = theme(
    'table',
    $header,
    $rows ? $rows : array(array(array('data' => t('No filters have been defined.'), 'colspan' => 5)))
  );
  
  return $table;
}

/*****************************************************************************
 * Theme functions.
 ****************************************************************************/

/**
 * Theme for draggable tree of replacement rules
 */
function theme_customfilter_rules($form) {
  $header = array(
    t('Name'),
    t('Description'),
    t('Match'),
    '',
  );
  
  $header[] = array('data' => t('Operations'), 'colspan' => 3);
  
  $rows = array();  
  foreach (element_children($form['rules']) as $rid) {
    $element =& $form['rules'][$rid];
    
    $element['prid']['#type'] = 'hidden';
    $element['weight']['#type'] = 'hidden';
    $element['prid']['#attributes']['class'] = 'rule-prid';
    $element['rid']['#attributes']['class'] = 'rule-rid';
    $element['weight']['#attributes']['class'] = 'rule-weight';
    
    $row['data'] = array(
      theme('indentation', $element['level']['#value']) . drupal_render($element['name']),
      drupal_render($element['description']),
      drupal_render($element['matches']),
      drupal_render($element['rid']) . drupal_render($element['prid']) . drupal_render($element['weight']),
    );
    
    if ($element['operations'])
      $row['data'] = array_merge(
        $row['data'],
        array(
          drupal_render($element['operations']['add']),
          drupal_render($element['operations']['edit']),
          drupal_render($element['operations']['delete']),
        )
      );
    
    $row['class'] = 'draggable';
    
    $rows[] = $row;
  }
  
  $output = theme('table', $header, $rows, array('id' => 'customfilter-rules-tree'));
  $output .= drupal_render($form);
  
  drupal_add_tabledrag('customfilter-rules-tree', 'match', 'parent', 'rule-prid', 'rule-prid', 'rule-rid', TRUE);
  drupal_add_tabledrag('customfilter-rules-tree', 'order', 'sibling', 'rule-weight', NULL, NULL, TRUE);
  
  return $output;
}

/*****************************************************************************
 * Private functions.
 ****************************************************************************/

/**
 * Return the rules tree form.
 */
function _customfilter_rules_tree_form($rules, $level = 0) {
  static $base = 'admin/settings/filters/customfilter';
  
  $form = array('#tree' => TRUE);
  
  foreach ($rules as $rule) {
    $fid = $rule['fid'];
    $rid = $rule['rid'];
    
    $form[$rid] = array(
      'name' => array(
        '#value' => l($rule['name'], "$base/$fid/$rid/edit"),
      ),
      'description' => array(
        '#value' => $rule['description'],
        '#prefix' => '<em>',
        '#suffix' => '</em>',
      ),
      'matches' => array(
        '#value' => ($rule['matches']? $rule['matches']: 'all'),
      ),
      'weight' => array(
        '#type' => 'weight',
        '#default_value' => $rule['weight'],
      ),
      'rid' => array(
        '#type' => 'hidden',
        '#value' => $rid,
      ),
      'prid' => array(
        '#type' => 'textfield',
        '#default_value' => $rule['prid'],
        '#size' => 4,
      ),
      'level' => array(
        '#type' => 'value',
        '#value' => $level,
      ),
    );
    
    $form[$rid]['operations'] = array(
      'add' => array(
        '#value' => l(t('add'), "$base/$fid/$rid/add"),
      ),
      'edit' => array(
        '#value' => l(t('edit'), "$base/$fid/$rid/edit"),
      ),
      'delete' => array(
        '#value' => l(t('delete'), "$base/$fid/$rid/delete"),
      ),
    );
    
    if ($rule['sub']) {
      _customfilter_rules_tree_form($rule['sub'], $level + 1);
    }
  }
  
  return $form;
}

/**
 * @} End of "addtogroup customfilter".
 */